@@ -9,24 +9,23 @@ import pyvista as pv
 from pyansys.common import (STRESS_TYPES, STRAIN_TYPES,
                             PRINCIPAL_STRESS_TYPES,
                             THERMAL_STRAIN_TYPES)
-from pyansys.rst import ResultFile, trans_to_matrix, check_comp
+from pyansys.rst import Result, check_comp
 from pyansys import _binary_reader
 
 np.seterr(divide='ignore', invalid='ignore')
 
 
-class CyclicResult(ResultFile):
+class CyclicResult(Result):
     """Adds cyclic functionality to the result class"""
 
     def __init__(self, filename):
-        """ Initializes object """
+        """Initializes cyclic result"""
         super().__init__(filename)
 
         # sanity check
-        if self.n_sector == 1:
+        if not self._is_cyclic:
             raise TypeError('Result is not a cyclic model')
 
-        self._animating = False
         self._rotor_cache = None
         self._has_duplicate_sector = None
         self._is_repeated_mode = np.empty(0)
@@ -108,6 +107,12 @@ class CyclicResult(ResultFile):
         mask_a = np.isclose(self.time_values, np.roll(self.time_values, 1))
         mask_b = np.isclose(self.time_values, np.roll(self.time_values, -1))
         self._is_repeated_mode = np.logical_or(mask_a, mask_b)
+
+        # edge case for single pair of repeated modes
+        if self._is_repeated_mode.size == 2 and self._is_repeated_mode.all():
+            self._repeated_index = np.array([1, 0])
+            return
+
         self._repeated_index = np.empty(self._is_repeated_mode.size, np.int)
         self._repeated_index[:] = -1
         if np.any(self._is_repeated_mode):
